require 'rest-client'

module Stargate
  module Client
    # Internal: A client implementation compatible with Inproc server engine.
    class Protocol::Inproc < Protocol
      RegistryNotFoundError = Class.new(::Stargate::Client::Error)

      def initialize(*)
        super
        @registry_version = ::Stargate::INPROC[uri.to_s]
        @caller = ::Stargate::Server::Caller.new(@registry_version)
        raise RegistryNotFoundError, "Local registry not found: #{uri.to_s}" unless @registry_version
      end

      def fetch_definitions
        unpack_definitions(@registry_version.definitions.map(&:serialize))
      end

      def call(klass, method, *args)
        unpack_payload(@caller.call(klass.remote_name, method, *args))
      rescue => err
        raise RemoteExecutionError.new(err.class.to_s, err.message)
      end

      register %w[inproc], self
    end
  end
end
