module Stargate
  module Codec
    # Public: Base codec error.
    Error = Class.new(::Stargate::Error)

    # Public: Raised when given codec is not registered.
    UndefinedCodecError = Class.new(Error)

    # Public: Raised when codec cannot decode given data string.
    DecodeError = Class.new(Error)

    @@codecs = {}

    # Internal: Returns all registered codecs.
    def self.codecs
      @@codecs
    end

    # Public: Register new codec.
    def self.register(codec)
      codecs[codec.content_type.to_s] = codec
    end

    # Public: Find codec by given identifier.
    def self.find_by_id(id)
      codecs.values.find { |codec| codec.id == id.to_sym } or raise UndefinedCodecError, "No such codec: #{id}"
    end

    # Public: Find codec for given content type.
    def self.[](content_type)
      codecs[content_type.to_s] or raise UndefinedCodecError, "No codec for type: #{content_type}"
    end

    require 'stargate/codec/json'
    require 'stargate/codec/bencode'
    require 'stargate/codec/message_pack'
  end
end
