module Stargate
  module Server
    # Internal: We need a way to call registered stuff. This little service handles it. Caller is
    # defined per registry version and operates only within its context.
    class Caller
      include Logging
      include Stargate::Marshal

      def initialize(registry_version)
        @marshaller       = Marshaller.new(registry_version)
        @registry_version = registry_version
      end

      def call(klass_name, method, *args)
        method, metadata = method.to_sym, @registry_version[klass_name]
        metadata.class_methods.include?(method) or raise MethodNotExposedError, "Method not exposed: #{klass_name}.#{method}"
        result = metadata.klass.send(method, *args)
        log.debug("Method call performed", class: klass_name, method: method, result:result.inspect)
        @marshaller.marshal(result)
      end
    end
  end
end
