module Stargate
  module Server
    # Public: Base server error.
    Error = Class.new(::Stargate::Error)

    # Public: Base class for not-found type errors.
    NotFoundError = Class.new(Error)

    # Public: Raised when trying to get non-existent registry version.
    RegistryVersionUndefinedError = Class.new(Error)

    # Public: Raised when requested class is not exposed/registered.
    ClassNotExposedError = Class.new(NotFoundError)

    # Public: Raised when called method is not exposed.
    MethodNotExposedError = Class.new(NotFoundError)

    # Public: Raised when trying to register the same class twice within single registry.
    ClassAlreadyRegisteredError = Class.new(Error)

    # Public: Raised when trying to return (and pack into payload) data of unregistered or unsupported type.
    # For example when exposed class method returns value of type different than itself, hash or basic.
    UnregisteredClassExposureError = Class.new(Error)
  end
end
