module Stargate
  module Server
    # Internal: Registry contains list of versions, each with defined served classes.
    #
    # Example:
    #
    #   Stargate::Server::Registry.new do
    #     version 1 do
    #       serve MyClass do
    #         class_methods :foo, :bar
    #         attributes :foo, :bar, :baz
    #       end
    #     end
    #   end
    #
    class Registry
      include Enumerable

      # Public: List of registry versions.
      attr_reader :versions

      # Public: Constructor.
      def initialize(&block)
        @versions = {}
        instance_eval(&block) if block_given?
      end

      # Public: Defines new version. Executes with internal DSL.
      def version(number, &block)
        @versions[number] ||= RegistryVersion.new(number, &block)
      end

      # Public: Returns given registry version.
      def [](version)
        versions[version] or raise RegistryVersionUndefinedError, "No such registry version: #{version}"
      end

      def each(&block)
        versions.each(&block)
      end
    end
  end
end
